@@ -4,7 +4,6 @@ pub mod v_text;
 pub use v_element::VElement;
 pub use v_text::VText;
 
-#[derive(Debug)]
 pub enum VNode {
     VElement(VElement),
     VText(VText),
@@ -20,3 +19,13 @@ impl VNode {
         }
     }
 }
+
+impl std::fmt::Debug for VNode {
+    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
+        match self {
+            Self::RNode(..) => write!(f, "[RNode]"),
+            Self::VText(text) => text.fmt(f),
+            Self::VElement(element) => element.fmt(f),
+        }
+    }
+}
